VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "SparseArray"
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private RowSentinel As New SparseArrayRow

' The value that represents an empty cell.
Private p_MissingValue As String
' Retuen the value used for missing cells.
Property Get MissingValue() As String
    MissingValue = p_MissingValue
End Property
' Set the value used for missing cells.
Property Let MissingValue(new_value As String)
    p_MissingValue = MissingValue
End Property
' Return a textual representation of the data.
Property Get TextValue() As String
Dim row_hdr As SparseArrayRow
Dim cell As SparseArrayCell
Dim txt As String

    Set row_hdr = RowSentinel.NextRow
    Do While Not (row_hdr Is Nothing)
        Set cell = row_hdr.CellSentinel.NextCol
        Do While Not (cell Is Nothing)
            txt = txt & "(" & _
                Format$(row_hdr.Row) & _
                ", " & _
                Format$(cell.Col) & _
                ") = " & cell.Value & vbCrLf
            Set cell = cell.NextCol
        Loop
        Set row_hdr = row_hdr.NextRow
    Loop
    
    TextValue = txt
End Property

' Get the value for Array(row, col).
Public Property Get Value(ByVal Row As Integer, ByVal Col As Integer) As String
Dim row_hdr As SparseArrayRow
Dim cell As SparseArrayCell
Dim no_value As Boolean

    ' Find the row.
    Set row_hdr = RowSentinel.NextRow
    Do
        If row_hdr Is Nothing Then Exit Do
        If row_hdr.Row >= Row Then Exit Do
        Set row_hdr = row_hdr.NextRow
    Loop
    
    ' See if we have the right row.
    If row_hdr Is Nothing Then
        no_value = True
    ElseIf row_hdr.Row > Row Then
        no_value = True
    Else
        no_value = False
    End If
    If no_value Then
        ' There's no such row. Return
        ' the missing value.
        Value = p_MissingValue
        Exit Property
    End If
    
    ' Find the column.
    Set cell = row_hdr.CellSentinel.NextCol
    Do
        If cell Is Nothing Then Exit Do
        If cell.Col >= Col Then Exit Do
        Set cell = cell.NextCol
    Loop
    
    ' See if we have the right column.
    If cell Is Nothing Then
        no_value = True
    ElseIf cell.Col > Col Then
        no_value = True
    Else
        no_value = False
    End If
    If no_value Then
        ' There's no such column in this row.
        ' Return the missing value.
        Value = p_MissingValue
        Exit Property
    End If
    
    Value = cell.Value
End Property
' Save the value for Array(row, col).
Public Property Let Value(ByVal Row As Integer, ByVal Col As Integer, ByVal new_value As String)
Dim row_hdr As SparseArrayRow
Dim nxt_row As SparseArrayRow
Dim cell As SparseArrayCell
Dim nxt_cell As SparseArrayCell
Dim no_value As Boolean
    
    ' Find the row before this one.
    Set row_hdr = RowSentinel
    Do
        Set nxt_row = row_hdr.NextRow
        
        If nxt_row Is Nothing Then Exit Do
        If nxt_row.Row >= Row Then Exit Do
        Set row_hdr = nxt_row
        Set nxt_row = row_hdr.NextRow
    Loop
    
    ' See if we have the right row.
    If nxt_row Is Nothing Then
        no_value = True
    ElseIf nxt_row.Row > Row Then
        no_value = True
    Else
        no_value = False
    End If
    
    ' If we don't have the row and the new
    ' value is not blank, add the row.
    If no_value Then
        If new_value = p_MissingValue Then _
            Exit Property
    
        ' Add a new row after row_hdr.
        Set nxt_row = New SparseArrayRow
        Set nxt_row.NextRow = row_hdr.NextRow
        Set row_hdr.NextRow = nxt_row
        nxt_row.Row = Row
    End If

    ' Now we are pointing to the right row.
    ' Find the right column.
    Set cell = nxt_row.CellSentinel
    Do
        Set nxt_cell = cell.NextCol
        If nxt_cell Is Nothing Then Exit Do
        If nxt_cell.Col >= Col Then Exit Do
        Set cell = nxt_cell
        Set nxt_cell = cell.NextCol
    Loop
    
    ' See if we have the right cell.
    If nxt_cell Is Nothing Then
        no_value = True
    ElseIf nxt_cell.Col > Col Then
        no_value = True
    Else
        no_value = False
    End If
    
    ' If we don't have the cell and the new
    ' value is not blank, add the cell.
    If no_value Then
        If new_value = p_MissingValue Then _
            Exit Property
    
        ' Add a new cell.
        Set nxt_cell = New SparseArrayCell
        Set nxt_cell.NextCol = cell.NextCol
        Set cell.NextCol = nxt_cell
        nxt_cell.Col = Col
    End If
    
    ' We now have the cell. Set its value.
    If new_value <> p_MissingValue Then
        ' Save the new value.
        nxt_cell.Value = new_value
    Else
        ' Delete the entry.
        Set cell.NextCol = nxt_cell.NextCol
    
        ' If there are no more items in the
        ' row, delete the row, too.
        If nxt_row.CellSentinel.NextCol Is Nothing Then
            Set row_hdr.NextRow = nxt_row.NextRow
        End If
    End If
End Property
